package example_parsing_numbers;

/**
 * Illustrates deciding if a string is an int, double using two approaches:
 * 
 * 1. try/catch - isIntegerParseInt, isDoubleParseDouble
 * 2. character inspection - isIntegerByCharacter
 * 
 * There a few odd cases.
 * 
 * @author dgibson
 *
 */

public class ParseNumbers {

	public static void main(String[] args) {
		String[] intsGood = {"-224", "-1", "-0", "0", "+0", "1", "224", "+100"};
		// Some of these are actually good: +0, +100
		String[] intsBad = {"- 224", "-1 ", "+0", "+100", "224 ", "a224", "22a4", "224a"};
		
		// Actually, these are bad: "+0", "+100"
		evaluateByCharacter(intsGood, "Should all be true:");
		
		evaluateParseInt(intsGood, "\nShould all be true:");
		
		evaluateByCharacter(intsBad, "\nShould all be false:");
		
		// Actually, these are good: +0, +100
		evaluateParseInt(intsBad, "\nShould all be false:");

		String[] dblsGood = {"-224", "-224.", "-224.2", "-0", "0", "1.55", "224.9923"};
		String[] dblsBad = {"a2", "2.2a2", "2.333   ", "  2.2"};
		
		evaluateDoubleByParseDouble(dblsGood, "\nShould all be true:");

		// Some of these are actually good: "2.333   ", "  2.2"
		evaluateDoubleByParseDouble(dblsBad, "\nShould all be false:");
	}
	
	public static void evaluateByCharacter(String[] vals, String title) {
		System.out.println(title);
		for(String s : vals) {
			boolean isInt = isIntegerByCharacter(s);
			String msg = String.format("isIntegerByCharacter(\"%s\")=%b", s, isInt);
			System.out.println(msg);
		}
	}

	public static void evaluateDoubleByParseDouble(String[] vals, String title) {
		System.out.println(title);
		for(String s : vals) {
			boolean isInt = isDoubleParseDouble(s);
			String msg = String.format("isDoubleParseDouble(\"%s\")=%b", s, isInt);
			System.out.println(msg);
		}
	}

	public static void evaluateParseInt(String[] vals, String title) {
		System.out.println(title);
		for(String s : vals) {
			boolean isInt = isIntegerParseInt(s);
			String msg = String.format("isIntegerParseInt(\"%s\")=%b", s, isInt);
			System.out.println(msg);
		}
	}

	public static boolean isIntegerParseInt(String str) {
		try {
			int x = Integer.parseInt(str);
			return true;
		}
		catch(NumberFormatException nfe) {
			return false;
		}
	}
	
	public static boolean isDoubleParseDouble(String str) {
		try {
			double x = Double.parseDouble(str);
			return true;
		}
		catch(NumberFormatException nfe) {
			return false;
		}
	}
	
	public static boolean isIntegerByCharacter(String str) {
	    if (str == null) {
	        return false;
	    }
	    int length = str.length();
	    if (length == 0) {
	        return false;
	    }
	    int beg = 0;
	    if (str.charAt(0) == '-') {
	        if (length == 1) {
	            return false;
	        }
	        beg = 1;
	    }
	    for (int i=beg; i < length; i++) {
	        char c = str.charAt(i);
	        if (c < '0' || c > '9') {
	            return false;
	        }
	    }
	    return true;
	}

}
